1#ifndef __MATH_MATH_KERNEL_H__
2#define __MATH_MATH_KERNEL_H__
50 for (
int i = idx;
i < n;
i +=
str) {
94 const int ly,
const int lz) {
95 const int idx2 = idx - 1;
96 index[3] =
idx2 / (lx * ly * lz);
97 index[2] = (
idx2 - (lx * ly * lz) * index[3]) / (lx * ly);
98 index[1] = (
idx2 - (lx * ly * lz) * index[3] - (lx * ly) * index[2]) / lx;
99 index[0] = (
idx2 - (lx * ly * lz) * index[3] - (lx * ly) * index[2]) -
109 const int n1,
const int n2,
const int nf) {
110 return ((
i) + (n1) * (((
j) - 1) + (n2) * (((
k) - 1) + (
nf) * (((l) - 1))))) - 1;
116template<
typename T >
132 for (
int m = idx; m <
n_mask; m +=
str) {
133 const int f = facet[m + 1];
157template<
typename T >
175template<
typename T >
190#if __CUDA_ARCH__ < 600
197template<
typename T >
207#if __CUDA_ARCH__ >= 600
208 for (
int i = idx;
i < m;
i +=
str)
212 assert(0 &&
"masked_atomic_reduction_kernel requires compute capability 6.0 or higher.");
219template<
typename T >
237template<
typename T >
260 const int mask_size) {
265 for (
int i = idx;
i < mask_size;
i +=
str) {
a[
mask[
i]] = c; }
271template<
typename T >
280 for (
int i = idx;
i < n;
i +=
str) {
288template<
typename T >
296 for (
int i = idx;
i < n;
i +=
str) {
304template<
typename T >
313 for (
int i = idx;
i < n;
i +=
str) {
321template<
typename T >
329 for (
int i = idx;
i < n;
i +=
str) {
337template<
typename T >
346 for (
int i = idx;
i < n;
i +=
str) {
354template<
typename T >
364 for (
int i = idx;
i < n;
i +=
str) {
372template<
typename T >
379 for (
int i = idx;
i < n;
i +=
str) {
387template<
typename T >
396 for (
int i = idx;
i < n;
i +=
str) {
404template<
typename T >
412 for (
int i = idx;
i < n;
i +=
str) {
420template<
typename T >
428 for (
int i = idx;
i < n;
i +=
str) {
436template<
typename T >
445 for (
int i = idx;
i < n;
i +=
str) {
453template<
typename T >
463 for (
int i = idx;
i < n;
i +=
str) {
464 a[
i] = b[
i] + c[
i] + d[
i];
471template<
typename T >
480 for (
int i = idx;
i < n;
i +=
str) {
488template<
typename T >
499 for (
int i = idx;
i < n;
i+=
str) {
502 tmp += p[
j][
i]*alpha[
j];
511template<
typename T >
520 for (
int i = idx;
i < n;
i +=
str) {
528template<
typename T >
537 for (
int i = idx;
i < n;
i +=
str) {
545template<
typename T >
556 for (
int i = idx;
i < n;
i +=
str) {
564template<
typename T >
577 for (
int i = idx;
i < n;
i +=
str) {
585template<
typename T >
600 for (
int i = idx;
i < n;
i +=
str) {
608template<
typename T >
616 for (
int i = idx;
i < n;
i +=
str) {
624template<
typename T >
632 for (
int i = idx;
i < n;
i +=
str) {
640template<
typename T >
649 for (
int i = idx;
i < n;
i +=
str) {
657template<
typename T >
665 for (
int i = idx;
i < n;
i +=
str) {
673template<
typename T >
682 for (
int i = idx;
i < n;
i +=
str) {
690template<
typename T >
699 for (
int i = idx;
i < n;
i +=
str) {
700 a[
i] =
a[
i] - b[
i] * c[
i];
707template<
typename T >
715 for (
int i = idx;
i < n;
i +=
str) {
723template<
typename T >
732 for (
int i = idx;
i < n;
i +=
str) {
740template<
typename T >
749 for (
int i = idx;
i < n;
i +=
str) {
750 a[
i] =
a[
i] + b[
i] * c[
i];
758template<
typename T >
768 for (
int i = idx;
i < n;
i +=
str) {
769 a[
i] =
a[
i] + b[
i] * c[
i] * d[
i];
777template<
typename T >
787 for (
int i = idx;
i < n;
i +=
str) {
788 a[
i] =
a[
i] + s * b[
i] * c[
i];
796template<
typename T >
809 for (
int i = idx;
i < n;
i +=
str) {
818template<
typename T >
833 for (
int i = idx;
i < n;
i +=
str) {
836 u3[
i] = v1[
i]*w2[
i] - v2[
i]*w1[
i];
884template<
typename T >
890 for (
int i = idx;
i<n ;
i +=
str)
915template<
typename T >
921 for (
int i = idx;
i<n ;
i +=
str)
946template<
typename T >
952 for (
int i = idx;
i<n ;
i +=
str)
978template<
typename T_acc >
989 for (
int i=idx ;
i<n ;
i+=step)
1013template<
typename T >
1028 for (
int i = idx;
i < n;
i+=
str) {
1048template<
typename T,
typename T_acc >
1063 for (
int i = idx;
i < n;
i+=
str) {
1083template<
typename T,
typename T_acc >
1098 for (
int i = idx;
i < n;
i+=
str) {
1124template<
typename T,
typename T_acc >
1138 for (
int i = idx;
i < n;
i+=
str) {
1159template<
typename T,
typename T_acc >
1173 for (
int i = idx;
i < n;
i+=
str) {
1194template<
typename T,
typename T_acc >
1207 for (
int i = idx;
i<n ;
i +=
str)
1229template<
typename T >
1243 for (
int i = idx;
i<n ;
i +=
str)
1265template<
typename T >
1279 for (
int i = idx;
i<n ;
i +=
str)
1301template<
typename T >
1308 for (
int i = idx;
i < n;
i +=
str) {
1320template <
typename T>
1334template <
typename T>
1349template <
typename T>
1362template <
typename T>
1369 for (
int i = idx;
i < n;
i +=
str)
a[
i] =
max(b[
i], c);
1376template <
typename T>
1390template <
typename T>
1405template <
typename T>
1418template <
typename T>
1425 for (
int i = idx;
i < n;
i +=
str)
a[
i] =
min(b[
i], c);
__global__ void ale_add_kinematics_kernel(const int n, T *__restrict__ wx, T *__restrict__ wy, T *__restrict__ wz, const T *__restrict__ x_ref, const T *__restrict__ y_ref, const T *__restrict__ z_ref, const T *__restrict__ phi, const T *__restrict__ x, const T *__restrict__ y, const T *__restrict__ z, const kinematics_params_t kin_params)
__global__ void const T *__restrict__ x
__global__ void const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ const T *__restrict__ w3
__global__ void addcol4_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const int n)
__global__ void cwrap_kernel(T *__restrict__ a, const T min_val, const T max_val, const int n)
__global__ void pwmin_vec3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void reduce_kernel(T *bufred, const int n)
__global__ void cdiv2_kernel(T *__restrict__ a, T *__restrict__ b, const T c, const int n)
__global__ void invcol2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void add2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void add4s3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const T c1, const T c2, const T c3, const int n)
__inline__ __device__ T reduce_warp(T val)
__global__ void masked_scatter_copy_aligned_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void masked_atomic_reduction_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int m)
__global__ void pwmax_vec3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void glsc2_kernel(const T *a, const T *b, T_acc *buf_h, const int n)
__global__ void addcol3s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T s, const int n)
__global__ void cfill_mask_kernel(T *__restrict__ a, const T c, const int size, int *__restrict__ mask, const int mask_size)
__global__ void cdiv_kernel(T *__restrict__ a, const T c, const int n)
__global__ void masked_copy_kernel_aligned(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void face_masked_gather_copy_kernel(T *__restrict__ a, const T *__restrict__ b, const int *__restrict__ mask, const int *__restrict__ facet, const int n1, const int n2, const int lx, const int ly, const int lz, const int n_mask)
__global__ void masked_gather_copy_aligned_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void pwmax_sca2_kernel(T *__restrict__ a, const T c, const int n)
__device__ __forceinline__ void face_gather_nonlinear_index(int *index, const int idx, const int lx, const int ly, const int lz)
__global__ void reduce_max_kernel(T *bufred, const T ninf, const int n)
__global__ void pwmin_vec2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void add3s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T c1, const T c2, const int n)
__inline__ __device__ T reduce_max_warp(T val)
__global__ void glsum_kernel(const T *a, T_acc *buf_h, const int n)
__global__ void add2s1_kernel(T *__restrict__ a, const T *__restrict__ b, const T c1, const int n)
__device__ __forceinline__ int face_gather_idx(const int i, const int j, const int k, const int l, const int n1, const int n2, const int nf)
__global__ void add5s4_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const T *__restrict__ e, const T c1, const T c2, const T c3, const T c4, const int n)
__global__ void masked_gather_copy_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void add2s2_many_kernel(T *__restrict__ x, const T **p, const T *alpha, const int p_cur, const int n)
__global__ void glsc3_many_kernel(const T *a, const T **b, const T *c, T_acc *buf_h, const int j, const int n)
__global__ void pwmax_vec2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void cmult_kernel(T *__restrict__ a, const T c, const int n)
__global__ void addcol3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void pwmin_sca3_kernel(T *__restrict__ a, const T *__restrict b, const T c, const int n)
__global__ void pwmax_sca3_kernel(T *__restrict__ a, const T *__restrict b, const T c, const int n)
__global__ void power_kernel(T *__restrict__ ap, const T *__restrict__ a, const T p, const int n)
__global__ void col2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void masked_copy_kernel_0(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void sqrt_inplace_kernel(T *__restrict__ a, const int n)
__global__ void col3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void sub2_kernel(T *__restrict__ a, const T *__restrict__ b, const int n)
__global__ void glmin_kernel(const T *a, const T pinf, T *buf_h, const int n)
__global__ void cmult2_kernel(T *__restrict__ a, T *__restrict__ b, const T c, const int n)
__global__ void pwmin_sca2_kernel(T *__restrict__ a, const T c, const int n)
__global__ void sub3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void glsubnorm2_kernel(const T *a, const T *b, T_acc *buf_h, const int n)
__global__ void glsc3_reduce_kernel(T_acc *bufred, const int n, const int j)
__global__ void add2s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T c1, const int n)
__global__ void vdot3_kernel(T *__restrict__ dot, const T *__restrict__ u1, const T *__restrict__ u2, const T *__restrict__ u3, const T *__restrict__ v1, const T *__restrict__ v2, const T *__restrict__ v3, const int n)
__global__ void invcol1_kernel(T *__restrict__ a, const int n)
__global__ void add3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void glmax_kernel(const T *a, const T ninf, T *buf_h, const int n)
__global__ void vlsc3_kernel(const T *a, const T *b, const T *c, T *buf_h, const int n)
__global__ void add4_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const T *__restrict__ d, const int n)
__global__ void cfill_kernel(T *__restrict__ a, const T c, const int n)
__global__ void masked_scatter_copy_kernel(T *__restrict__ a, T *__restrict__ b, int *__restrict__ mask, const int n, const int n_mask)
__global__ void glsc3_kernel(const T *a, const T *b, const T *c, T_acc *buf_h, const int n)
__global__ void reduce_min_kernel(T *bufred, const T pinf, const int n)
__global__ void vcross_kernel(T *__restrict__ u1, T *__restrict__ u2, T *__restrict__ u3, const T *__restrict__ v1, const T *__restrict__ v2, const T *__restrict__ v3, const T *__restrict__ w1, const T *__restrict__ w2, const T *__restrict__ w3, const int n)
__global__ void addsqr2s2_kernel(T *__restrict__ a, const T *__restrict__ b, const T c1, const int n)
__global__ void cadd2_kernel(T *__restrict__ a, T *__restrict__ b, const T c, const int n)
__global__ void absval_kernel(T *__restrict__ a, const int n)
__global__ void invcol3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void subcol3_kernel(T *__restrict__ a, const T *__restrict__ b, const T *__restrict__ c, const int n)
__global__ void cadd_kernel(T *__restrict__ a, const T c, const int n)
__inline__ __device__ T reduce_min_warp(T val)
Object for handling masks in Neko.